(ns cipher-analytical-machine.ciphers.simple-substitution
  (:require [clojure.string :as cs]
            [clojure.set :as set]
            [cipher-analytical-machine.ciphers.caesar :as caesar])
  (:gen-class))

(defn shuffled-numbers
  "Generate the shuffled order for integer list."
  [size]
  (-> size
      (take (range))
      (shuffle)))

(defn generate-substitution-table
  "Generate the map (char, int) for the substittion."
  [symbols]
  (->> (count symbols)
       (shuffled-numbers)
       (zipmap symbols)))

(defn find-value-in-table
  "It uses the substitution table to find the value of a char or a number."
  [char substitution-table symbols]
  (get substitution-table char))

(defn encrypt-message
  "Encrypt a message using the simple substitution cipher. The function is case-insensitive. If a symbol isn't in the symbols list, then it will be removed."
  [message key substitution-table symbols]
  (let [max-index (count symbols)]
    (->> message
         (cs/lower-case)
         (caesar/encrypt-message key symbols)
         (map (fn [char] (find-value-in-table char substitution-table symbols)))
         (cs/join \,))))

(defn decrypt-message
  "Decrypt a message using the simple substitution cipher. The function is case-insensitive."
  [message key substitution-table symbols]
  (let [substitution-table (set/map-invert substitution-table)
        max-index (count symbols)
        message (cs/split message #",")]
      (->> message
           (map #(Integer/parseInt %))
           (map (fn [char] (find-value-in-table char substitution-table symbols)))
           (caesar/encrypt-message key symbols)
           (cs/join))))

